﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  [StorableClass]
  [Item("CombinedIntegerVectorComparer", "")]
  public class CombinedIntegerVectorComparer : Item, IEqualityComparer<CombinedIntegerVector>, IClassifierComparer {

    [StorableConstructor]
    protected CombinedIntegerVectorComparer(bool deserializing) : base(deserializing) { }
    protected CombinedIntegerVectorComparer(CombinedIntegerVectorComparer original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CombinedIntegerVectorComparer(this, cloner);
    }
    public CombinedIntegerVectorComparer() : base() { }

    public bool Equals(CombinedIntegerVector x, CombinedIntegerVector y) {
      var xEnumerator = x.GetEnumerator();
      var yEnumerator = y.GetEnumerator();
      //xMove and yMove are used to check if both enumerators moved
      bool xMove = xEnumerator.MoveNext();
      bool yMove = yEnumerator.MoveNext();
      while (xMove && yMove && xEnumerator.Current == yEnumerator.Current) {
        xMove = xEnumerator.MoveNext();
        yMove = yEnumerator.MoveNext();
      }
      return !xMove && !yMove;
    }

    public int GetHashCode(CombinedIntegerVector obj) {
      int result = obj.ActionLength;
      foreach (var value in obj) {
        result ^= value;
      }
      return result.GetHashCode();
    }

    public bool Equals(IAction x, IAction y) {
      var xCast = x as CombinedIntegerVector;
      var yCast = y as CombinedIntegerVector;
      return x != null && y != null && Equals(xCast, yCast);
    }

    public int GetHashCode(IAction obj) {
      var objCast = obj as CombinedIntegerVector;
      return objCast != null ? GetHashCode(objCast) : obj.GetHashCode();
    }
  }
}
